// decoder.h (C) by Mats Sjoevall

#ifndef __DECODER_H__
#define __DECODER_H__

#include <sys/types.h>

#include "buf.h"
#include "id.h"

typedef struct {
// init codec
	int (*init)		(buf_t *buf, char *output_path);
// exit codec
	int (*exit)		();
// open codec
	int (*open)		();
// close codec
	int (*close)		();

// DENT: move buffer out
        int (*buf_init)         (buf_t *buf);
        int (*buf_exit)         (buf_t *buf);
// read codec
        int (*read)             (buf_t *buf, buf_entry_t *buf_entry);
// write codec
        int (*write)            ();

// set operational status (PLAY, PAUSE, STOP)
	int (*set_status)	(u_int stats);
// set attributes (audio, video, spu)
	int (*set_attributes)	(u_int id, void *attr);
} plugin_codec_t;



// audio attributes
typedef struct {
	int mode;		// 0=ac3, 1=mpeg, 2=lpcm
	int bits;		// 0=16bit, 1=20bitm 2=24bit
	int freq; 		// 0=44.1kHz 1=48kHz 2=96kHz 3=22.05kHz 4=32kHz
	int stream;		// 0-7
	int volume;		// 0-100
} plugin_codec_attr_audio_t;

// video attributes
typedef struct {
	int mode;		// 0=VOB, 1=VCD, 2=MPG
	int fps;		// 0=50Hz, 1=60Hz	
	int aspect_mode;	// 0=NORMAL, 1=PANSCAN, 2=LETTERBOX
	int aspect_output;	// 0=4:3, 3=16:9
	int tv_system;		// 0=NTSC, 1=NTSC60, 2=PAL, 3=PALM, 4=PALM60, 5=PALN, 6=PALNc, 7=PAL60
} plugin_codec_attr_video_t;


typedef struct {
				// DENT->MATS: i'd like to use -1 for off (like for audio and video)
	int num;		// number of SPU 0-31, 63=off
	char *clut;		// Color Lookup Table (palette)
} plugin_codec_attr_spu_t;

int DecoderSetStatus (u_int status);

// set video audio or subpicture attributes
int DecoderSetAttributes (u_int id, void *attr);





#define STATUS_STOP             0
#define STATUS_PAUSE            1
#define STATUS_PAUSE_ON         1
#define STATUS_PAUSE_OFF        2
#define STATUS_PLAY             3
#define STATUS_FINISH           4














int DecoderInit (buf_t *buf, char *output_path);
int DecoderPlay (void);
int DecoderStop (void); // Stop player and clear video
int DecoderPause (void);
/* Set subpicture palette */
int DecoderSPUCLUT (char *data);

/* n=-1 disable subpicture */
/* n=0-31 enable subpicture n */
int DecoderSPU (int n);

//	/* CSS authorization stuff */
//	int DecoderCSSDiscKey (char *data);
//	int DecoderCSSTitleKey (char *data, int cpm, int cp_sec, int cgms);
//	int DecoderCSSDriveAuthChallenge (char *data);
//	int DecoderCSSDriveAuthResponse (char *data);
//	int DecoderCSSHostAuth (char *data);

/* vol : 0-100 */
int DecoderAudioVolume (int vol);

#endif
