
/*
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>

#define SINGLE_BUFFER
#include <plugin/buf.h>


/**
 *
 */

int DecoderBufInit (buf_t *buf)
{
	int i;

	pthread_mutex_init(&(buf->ctrl.mutex_slots_left), NULL);
	pthread_cond_init (&(buf->ctrl.cond_slots_left), NULL);
	pthread_mutex_init(&(buf->ctrl.mutex_new_data), NULL);
	pthread_cond_init (&(buf->ctrl.cond_new_data), NULL);
	buf->ctrl.forbid_write = 0;

	for (i=0; i<BUF_DEPTH; i++) {
		buf->entry[i].mem_len = 0;
		buf->entry[i].data_len = 0;
		buf->entry[i].mem = NULL;
		buf->entry[i].data = NULL;
	}

	return 0;
}


/**
 *
 */

int DecoderBufExit (buf_t *buf)
{
	DecoderBufFlush (buf, BUF_ANY);

	return 0;
}


/**
 * Ever used a toilet? - then you should be familiar with the concept
 * behind this function
 */

void DecoderBufFlush (buf_t *buf, u_int buf_id)
{
	buf_entry_t entry;

	while (DecoderBufGet (buf, buf_id, &entry))
		free (entry.data);
}


// simple implementation of circular queues
/**
 *
 */

int DecoderBufPut (buf_t *buf, buf_entry_t *buf_entry)
{
	u_int write_index; 

	memcpy (&(buf->entry[buf->ctrl.write]), buf_entry, sizeof (buf_entry_t));

	write_index = (buf->ctrl.write + 1) % BUF_DEPTH;

	if (write_index == buf->ctrl.read) { // buffer full
		buf->ctrl.forbid_write = 1;
		pthread_cond_wait (&(buf->ctrl.cond_slots_left), &(buf->ctrl.mutex));
	}

	buf->ctrl.write = write_index;

// DENT move this out of the 'main stream' - i don't like to send this 
//	signal each time a new entry is made.	 
// inform listeners, we've got something for them
	pthread_cond_signal (&(buf->ctrl.cond_new_data));

	return 0;
}


/**
 *
 */

int DecoderBufGet (buf_t *buf, u_int buf_id, buf_entry_t *entry)
{
	if (buf->ctrl.read ==  buf->ctrl.write)	// no new entry
	  {
#if 1
	    printf(".");
	    fflush(stdout);
#endif
		return -1;
	  }

	memcpy (entry, &(buf->entry[buf->ctrl.read]), sizeof(buf_entry_t));

	buf->ctrl.read = (buf->ctrl.read + 1) % BUF_DEPTH;

	if (buf->ctrl.forbid_write) {
		u_int delta = (buf->ctrl.read - buf->ctrl.write) > 0  ?
			(buf->ctrl.read - buf->ctrl.write) :
			(BUF_DEPTH - buf->ctrl.write) + buf->ctrl.read;

// wait a little bit, until we allow reading again ...
// (high water = 100%, low water = 50%)

		if (delta > BUF_DEPTH/2) {
			buf->ctrl.forbid_write = 0;
			pthread_cond_signal (&(buf->ctrl.cond_slots_left));
		}
	}

	return 0;
}
