
/*
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


//#define USE_MPEG2DEC


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

//for test purposes
#include <fcntl.h>

#include <plugin/plugin.h>

#ifdef USE_MPEG2DEC
#include "config.h"
#include "old_crap.h"
#include "mpeg2.h"
#include "mpeg2_internal.h"

#include "mb_buffer.h"
#include "motion_comp.h"
#include "bitstream.h"
#include "idct.h"
#include "parse.h"
#include "display.h"
#include "inv_quantize.h"

#endif

static struct {
	pthread_t loopVideo;
	pthread_t loopAudio;

	int end_loop;
#ifndef USE_MPEG2DEC
	int fd_video;
#endif
	int fd_audio;
	buf_t *buf;
} decoder;


static plugin_codec_t codec_sw = {
	DecoderInit,
	NULL,
	NULL,
	NULL,
	DecoderBufInit,
	DecoderBufExit,
	DecoderBufPut,	// read
	NULL,		// write
	DecoderSetStatus,
	DecoderSetAttributes,
}; 


/**
 *
 */

plugin_codec_t *codec_init ()
{
	return &codec_sw;
}


#ifdef USE_MPEG2DEC
/**
 *
 */

void fill_buffer (uint_32 **data_start, uint_32 **data_end)
{
	static buf_entry_t buf_entry;

	if (buf_entry.mem)
		free (buf_entry.mem);

	while (DecoderBufGet (decoder.buf, BUF_VIDEO, &buf_entry) < 0) {
//		pthread_cond_wait (&decoder.buf->ctrl[BUF_VIDEO].cond_new_data, &decoder.buf->ctrl[BUF_VIDEO].mutex_new_data);
	}

	*data_start = (uint_32 *) (buf_entry.data);
	*data_end = (uint_32 *) (buf_entry.data+buf_entry.data_len);
}
#endif

/**
 *
 */

static void *_DecoderLoopVideo (void *foo)
{
#ifndef USE_MPEG2DEC
	buf_entry_t buf_entry;
#endif

	pthread_setcancelstate (PTHREAD_CANCEL_DEFERRED, NULL);

#ifdef USE_MPEG2DEC
	bitstream_init (fill_buffer);
	mpeg2_init ();
#endif
fprintf (stderr, "A"); 
	while (!decoder.end_loop) {
#ifdef USE_MPEG2DEC
fprintf (stderr, "B"); 
		mpeg2_decode ();		
#else
		if (!DecoderBufGet (decoder.buf, BUF_VIDEO, &buf_entry)) {
			if (decoder.fd_video >= 0)
				write (decoder.fd_video, buf_entry.data, buf_entry.data_len);
			free (buf_entry.mem);
		} else {
			pthread_cond_wait (&(decoder.buf->ctrl[BUF_VIDEO].cond_new_data), &(decoder.buf->ctrl[BUF_VIDEO].mutex_new_data));
		}
#endif
	}
fprintf (stderr, "C"); 

	return NULL;
}


/**
 *
 */

static void *_DecoderLoopAudio (void *foo)
{
	buf_entry_t buf_entry;

	pthread_setcancelstate (PTHREAD_CANCEL_DEFERRED, NULL);

	while (!decoder.end_loop) {
		if (!DecoderBufGet (decoder.buf, BUF_AUDIO, &buf_entry)) {
			if (decoder.fd_audio >= 0)
				write (decoder.fd_audio, buf_entry.data, buf_entry.data_len);
			free (buf_entry.mem);
		} else {
			pthread_cond_wait (&(decoder.buf->ctrl[BUF_AUDIO].cond_new_data), &(decoder.buf->ctrl[BUF_AUDIO].mutex_new_data));
		}
	}

	return NULL;
}


/**
 *
 */

int DecoderInit (buf_t *_buf, char *output_path)
{
	decoder.buf = _buf;
	
	fprintf (stderr, "DecoderInit\n");
#ifdef USE_MPEG2DEC
#else
	if ((decoder.fd_video = open ("/tmp/video", O_WRONLY)) < 0)
		fprintf (stderr, "WARNING: no /tmp/video found\n");
#endif
	if ((decoder.fd_audio = open ("/tmp/audio", O_WRONLY)) < 0)
		fprintf (stderr, "WARNING: no /tmp/audio found\n");

	return 0;
}


/**
 *
 */

int DecoderSetStatus (u_int status)
{
	switch (status) {
	case STATUS_PLAY:
		decoder.end_loop = 0;

		pthread_create (&decoder.loopVideo, NULL, _DecoderLoopVideo, NULL);
		pthread_create (&decoder.loopAudio, NULL, _DecoderLoopAudio, NULL);
		break;
	case STATUS_STOP:
	case STATUS_PAUSE:
		decoder.end_loop = 1;

		pthread_cancel (decoder.loopAudio);
		pthread_join (decoder.loopAudio, NULL);
		pthread_cancel (decoder.loopVideo);
		pthread_join (decoder.loopVideo, NULL);

		break;
	default:
		fprintf (stderr, "DecoderSetStatus: unhandled status %d\n", status);
		return -1;
	}

	return 0;
}


/**
 *
 */

int DecoderSetAttributes (u_int id, void *attr)
{
	return 0;
}

#if 0

/* CSS authorization stuff */
int DecoderCSSDiscKey (char *data)
{
	return 0;
}

int DecoderCSSTitleKey (char *data, int cpm, int cp_sec, int cgms)
{
	return 0;
}

int DecoderCSSDriveAuthChallenge (char *data)
{
	return 0;
}

int DecoderCSSDriveAuthResponse (char *data)
{
	return 0;
}

int DecoderCSSHostAuth (char *data)
{
	return 0;
}
#endif
