#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

#include "plugin/plugin.h"
#include "plugin/nav.h"
#include "nav.h"
#include "../input/input_vcd.h"

static int vcdOpen (plugin_t *plugin, char *path);
static int vcdPartPlayAutoStop (int title, int chapter, int parts2play);
static int vcdRegisterCb (u_int id, void (*cb) (void *, int ), void *data);
static int vcdUnregisterCb (u_int id);
static nav_tree_t *vcdGetInfo (plugin_t *nav);

static struct {
	char *root;
	plugin_t *nav;
} vcd_priv;

static plugin_nav_t vcd_nav = {
	vcdOpen,
	NULL,
	vcdPartPlayAutoStop,
	vcdGetInfo,
	vcdRegisterCb,		// register callback
	vcdUnregisterCb,
	&vcd_priv
};


/**
 *
 */

plugin_nav_t *plugin_init (void *foo)     
{     
        return &vcd_nav;     
}     


/**
 *
 */

void plugin_exit (void)
{
}


/**
 *
 */

static int vcdOpen (plugin_t *plugin, char *path)
{
	vcd_priv.root = path;
	vcd_priv.nav = plugin;

	if (pluginOpen (plugin, PLUGIN_ID_INPUT, DL_INPUT_VCD, path))
		return -1;

	if (pluginOpen (plugin, PLUGIN_ID_DECAPS, DL_DECAPS_VCD, NULL))
		return -1;

	PLUGIN_OPS_DECAPS(plugin)->blocksize = PLUGIN_OPS_INPUT(plugin)->blocksize;
	PLUGIN_OPS_DECAPS(plugin)->read = PLUGIN_OPS_INPUT(plugin)->read;
	if (PLUGIN_OPS_CODEC(plugin))
		PLUGIN_OPS_DECAPS(plugin)->write = PLUGIN_OPS_CODEC(plugin)->read;

//	PLUGIN_OPS_INPUT(plugin)->seek (0);
	PLUGIN_OPS_INPUT(plugin)->open ();

	PLUGIN_OPS_NAV(plugin) = &vcd_nav;

        return DVD_PLAYER_OK;
}


/**
 *
 */

static int vcdPartPlayAutoStop (int title, int chapter, int parts2play)
{
	plugin_codec_attr_audio_t attr_audio = {1, 0, 0, 0, -1};
	plugin_codec_attr_video_t attr_video = {1, 0, 1, -1, 2};

	PLUGIN_OPS_INPUT(vcd_priv.nav)->seek (chapter);	

//VCD MPEG,50Hz, PAL
	PLUGIN_OPS_CODEC(vcd_priv.nav)->set_attributes (BUF_VIDEO, &attr_video);
//MPEG,16bit,44.1kHz
	PLUGIN_OPS_CODEC(vcd_priv.nav)->set_attributes (BUF_AUDIO, &attr_audio);

	return DVD_PLAYER_OK;
}


/**
 *
 */

static int vcdRegisterCb (u_int id, void (*cb) (void *, int ), void *data)
{
	fprintf (stderr, "UI registered callback\n");

	switch (id) {
	case CB_CHANGE:
		((vcd_priv_t*)(PLUGIN_OPS_INPUT(vcd_priv.nav)->priv))->callb.chapter = cb;
		((vcd_priv_t*)(PLUGIN_OPS_INPUT(vcd_priv.nav)->priv))->callb.chapter_data = data;
		break;
	}

	return 0;
}


/**
 *
 */

static int vcdUnregisterCb (u_int id)
{
	return 0;
}


/**
 *
 */

#define TRACK_NAME_LEN (6 + 3 + 1)
static nav_tree_t *vcdGetInfo (plugin_t *nav)
{
	int i;
	int tracks;
	nav_tree_t *tree;

	tracks=((vcd_priv_t*)(PLUGIN_OPS_INPUT(vcd_priv.nav)->priv))->total_tracks;
	tree = malloc (sizeof (nav_tree_t));

        tree->num = 1;
        tree->title = malloc (sizeof (nav_subtree_t) * tree->num);

	{
		nav_subtree_t *sub_tree = tree->title;
		char *name;

		sub_tree->sub = (nav_tree_data_t *) malloc (sizeof
		(nav_tree_data_t) * tracks);

		sub_tree->num = tracks;
		sub_tree->name = "VCD";
		for (i=0; i < tracks; i++) {
			nav_tree_data_t *data = sub_tree->sub+i;
			name = (char *) malloc (TRACK_NAME_LEN);
			snprintf (name, TRACK_NAME_LEN, "Track %d",i);
			data->name = name;
		}
	}

	return tree;
}
