/*
 * subpic_decode.c - converts DVD subtitles to an XPM image
 *
 * Mostly based on hard work by:
 *
 * Copyright (C) 2000   Samuel Hocevar <sam@via.ecp.fr>
 *                       and Michel Lespinasse <walken@via.ecp.fr>
 *
 * Lots of rearranging by:
 *   Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *                                                     
 */

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

typedef unsigned int uint_32;
typedef unsigned char uint_8;
#include "display.h"


int fd;
int packet_size;
int id; /* 0: up  1: down */
int aligned;
int offset[2];
uint_8 palette[4];
uint_32 palette_to_rgb[16] = 
{
	0x00505050,
	0x00ff0000,
	0xffffffff,
	0x00000000,
	0x00005555,
	0x0000ff00,
	0x00cc2255,
	0x00cc0055,
	0x00404040,
	0x00202020,
	0x00b0b0b0,
	0x00d0d0d0,
	0x60606000,
	0x70707000,
	0x80808000,
	0x60606000,
};


unsigned char trans[4];
unsigned char buffer[10 * 1024];

int width = 0;
int height = 0;
int data_size;

unsigned int get_byte (void)
{
	unsigned char byte;

	byte = buffer[offset[id]];
	offset[id]++;

	return byte;
}

int get_nibble (void)
{
	static int next;

	if (aligned)
	{
		next = get_byte ();
		aligned = 0;
		return next >> 4;
	}
	else
	{
		aligned = 1;
		return next & 0xf;
	}
}

uint_32 x;
uint_32 x_org;
uint_32 y;
uint_32 y_org;
uint_32 display_time;


uint_32 *image;

void initialize(void)
{
	image = malloc(720 * 576 * sizeof(uint_32));

	display_init();

#if 0

	/* FIXME : the palette isn't accurate yet,
	 *         and we don't handle transparency
	 */
	printf("/* XPM */\n");
	printf("static char *foo[] = {\n");
	printf("/* columns rows colors chars-per-pixel */\n");
	printf("\"%i %i 16 1\",\n", width, height);
	printf("\"0 c grey60\",\n");
	printf("\"1 c red\",\n");
	printf("\"2 c white\",\n");
	printf("\"3 c black\",\n");
	printf("\"4 c yellow\",\n");
	printf("\"5 c green\",\n");
	printf("\"6 c magenta\",\n");
	printf("\"7 c cyan\",\n");
	printf("\"8 c grey10\",\n");
	printf("\"9 c grey30\",\n");
	printf("\"a c grey40\",\n");
	printf("\"b c grey50\",\n");
	printf("\"c c grey60\",\n");
	printf("\"d c grey70\",\n");
	printf("\"e c grey80\",\n");
	printf("\"f c grey60");
#endif
}	

void draw_pixels(length, colorid)
{
	while (length--)
		image[x++ + y * 720] = palette_to_rgb[palette[colorid]];
}

void next_line(void)
{
	draw_pixels(width - (x - x_org), 0);
	x = x_org;
	y++;
	id = 1 - id;
}

void terminate(void)
{
	while (y < (height + y_org))
		next_line();
}

void parse_subpic_header(void)
{
	unsigned char dummy;
	int i;

	//parse off the padding bytes. this should be done in extract_subpic
	do 
	{
		read(fd, &dummy, 1); 
	}
	while(dummy == 0xff);
	
	packet_size = dummy;
	read(fd, &dummy, 1);
	packet_size = (packet_size << 8) + dummy;

	lseek (fd, -2, SEEK_CUR);
	//XXX packet_size + 6 because we skip the mpeg stuffing info
	//0x000001be xxxx
	i = read (fd, buffer, packet_size+6);

	data_size = (((unsigned int)buffer[2]) << 8) + buffer[3];

	fprintf(stderr,"(subpic) packet_size = %d data_size = %d\n",packet_size,data_size);

	/* parse the header */

	i = data_size + 4;
	while (i < packet_size)
	{
		dummy = buffer [i];
		switch (dummy)
		{
			case 0x01: /* null packet ? */
			i++;
			break;

			case 0x02: /* 02 ff (ff) is the end of the packet */
			i = packet_size;
			break;

			case 0x03: /* palette */
			palette[0] = buffer [i+1] >> 4;
			palette[1] = buffer [i+1] & 0xf;
			palette[2] = buffer [i+2] >> 4;
			palette[3] = buffer [i+2] & 0xf;
			fprintf(stderr,"         palette [%d %d %d %d]\n",palette[0],palette[1],
					palette[2], palette[3]);
			i += 3;
			break;

			case 0x04: /* transparency palette */
			trans[0] = buffer [i+1] >> 4;
			trans[1] = buffer [i+1] & 0xf;
			trans[2] = buffer [i+2] >> 4;
			trans[3] = buffer [i+2] & 0xf;
			fprintf(stderr,"         trans [%d %d %d %d]\n",trans[0],trans[1],
					trans[2],trans[3]);
			i += 3;
			break;
			
			case 0x05: /* image coordinates */
			x_org = (((unsigned int)buffer[i+1]) << 4) + (buffer[i+2] >> 4);
			y_org = (((unsigned int)buffer[i+4]) << 4) + (buffer[i+5] >> 4);
			x = x_org;
			y = y_org;

			width = 1 + ( ((buffer[i+2] & 0x0f) << 8) + buffer[i+3] ) - x_org;
			height = 1 + ( ((buffer[i+5] & 0x0f) << 8) + buffer[i+6] ) - y_org;
				
			fprintf(stderr,"         x = %d y = %d width = %d height = %d\n",x,y,width,height);
			i += 7;
			break;

			case 0x06: /* image 1 / image 2 offsets */
			offset[0] = (((unsigned int)buffer[i+1]) << 8) + buffer[i+2];
			offset[1] = (((unsigned int)buffer[i+3]) << 8) + buffer[i+4];
			fprintf(stderr,"         field_1_offset = %d field_2_offset = %d\n",offset[0],
					offset[1]);
			i += 5;
			break;

			case 0xff: /* "ff xx yy zz uu" with 'zz uu' == start of control packet
			* I don't know yet what xx and yy stand for
			*/
			if ((buffer[i+3] != buffer[data_size+2]) || 
					(buffer[i+4] != buffer[data_size+3]) )
			{
				fprintf(stderr, "invalid control header (%.2x%.2x != %.2x%.2x) !\n",
				buffer[i+3], buffer[i+4], buffer[data_size+2], buffer[data_size+3] );
				//exit(1);
			}
			display_time = (buffer[i+1] << 8) + buffer[i+2];
			fprintf(stderr,"         display_time = %d = %f s\n", display_time, display_time / 30.0); 
			i += 5;
			break;

			default:
			fprintf(stderr, "invalid sequence in control header (%.2x) !\n", dummy);
			i++;
			//exit(1);
		}
	}

	id = 0;
	aligned = 1;
}
void
parse_subpic_data(void)
{
	while ((offset[1] < data_size + 2))
	{
		int code;

		code = get_nibble ();
		if (code >= 0x4)	/* 4 .. f */
		{
found_code:
			draw_pixels(code >> 2, code & 3);
			if (x >= width + x_org)
			{
				if (!aligned)
					get_nibble ();
					next_line ();
			}
			continue;
		}

		code = (code << 4) + get_nibble();
		if (code >= 0x10)	/* 1x .. 3x */
			goto found_code;

		code = (code << 4) + get_nibble();
		if (code >= 0x40)	/* 04x .. 0fx */
		{
			//if (aligned) fprintf (stderr, "0%x at odd offset\n", code); /* 0 xx */
			goto found_code;
		}

		code = (code << 4) + get_nibble();
		if (code >= 0x100)	/* 01xx .. 03xx */
		{
			//if (!aligned) fprintf (stderr, "0%x at odd offset\n", code); /* 0 xx x */
			goto found_code;
		}

		/* 00xx - should only happen for 00 00 */
		if (!aligned)
			code = (code << 4) + get_nibble(); /* 0 0x xx */

		if (code)
		{
			fprintf (stderr, "got unknown code 00%x (offset %x side %x, x=%d, y=%d)\n", code, offset[id], id, x, y);
			next_line ();
			continue;
		}

		/* aligned 00 00 */
		next_line ();
	}
}

int main (int argc, char *argv[])
{
	if (argc != 2)
	{
		printf("usage: %s file\n", argv[0]);
		exit (1);
	}

	fd = open (argv[1], 0);

	initialize();

	while(1)
	{
		parse_subpic_header();
		parse_subpic_data();
		terminate();

		display_frame(image);
		sleep(1);
		//uncomment for the real delay
		//usleep(display_time * 33333);
		memset(image,0,720 * 576 * sizeof(uint_32));
	}

	return 0;
}


